package be.jeffcheasey88.peeratcode.routes.users;

import static be.jeffcheasey88.peeratcode.framework.RequestType.POST;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;
import java.util.Base64.Encoder;
import java.util.regex.Matcher;

import javax.net.ssl.HttpsURLConnection;

import org.json.simple.JSONAware;
import org.json.simple.JSONObject;

import be.jeffcheasey88.peeratcode.bonus.extract.RouteDoc;
import be.jeffcheasey88.peeratcode.framework.HttpReader;
import be.jeffcheasey88.peeratcode.framework.HttpWriter;
import be.jeffcheasey88.peeratcode.framework.Route;
import be.jeffcheasey88.peeratcode.framework.Router;
import be.jeffcheasey88.peeratcode.framework.User;
import be.jeffcheasey88.peeratcode.repository.DatabaseRepository;
import be.jeffcheasey88.peeratcode.utils.FormResponse;

public class Register extends FormResponse {

	private DatabaseRepository databaseRepo;
	private Router router;
	private String usersFilesPath;
	private KeyPairGenerator generator;
	private Encoder encoder;
	private String gitToken;

	public Register(DatabaseRepository databaseRepo, Router router, String initUsersFilesPath, String gitToken){
		this.databaseRepo = databaseRepo;
		this.router = router;
		this.usersFilesPath = initUsersFilesPath;
		this.gitToken = gitToken;
		try {
			generator = KeyPairGenerator.getInstance("RSA");
			generator.initialize(2048); //a changer ?
		} catch (NoSuchAlgorithmException e){
			e.printStackTrace();
		}
		encoder = Base64.getEncoder();
	}
	
	@RouteDoc(path = "/register", responseCode = 200, responseDescription = "L'utilisateur est inscrit")
	@RouteDoc(responseCode = 403, responseDescription = "L'utilisateur est connecté")
	@RouteDoc(responseCode = 400, responseDescription = "Aucune données fournie / données invalide")

	@Route(path = "^\\/register$", type = POST)
	public void exec(Matcher matcher, User user, HttpReader reader, HttpWriter writer) throws Exception{
		if (user != null){
			writer.response(403, "Access-Control-Allow-Origin: *");
			return;
		}
		JSONObject json = json(reader);
		if(!areValids("pseudo","email","passwd","firstname","lastname")){
			writer.response(400, "Access-Control-Allow-Origin: *");
			return;
		}
		
		String pseudo = (String) json.get("pseudo");
		String email = (String) json.get("email");
		String password = (String) json.get("passwd");
		String firstname = (String) json.get("firstname");
		String lastname = (String) json.get("lastname");
		String description = (String) json.get("description");
		String group = (String) json.get("sgroup");
		String avatar = (String) json.get("avatar");

		boolean pseudoAvailable = databaseRepo.checkPseudoAvailability(pseudo);
		boolean emailAvailable = databaseRepo.checkEmailAvailability(email);
		if(pseudoAvailable && emailAvailable){
			int id;
			if((id = databaseRepo.register(pseudo, email, password, firstname, lastname, description, group,
					avatar)) >= 0) {
				writer.response(200,
						"Access-Control-Allow-Origin: *",
						"Access-Control-Expose-Headers: Authorization",
						"Authorization: Bearer " + this.router.createAuthUser(id));
				createFolderToSaveSourceCode(pseudo);
			}else{
				writer.response(400, "Access-Control-Allow-Origin: *");
			}
		}else{
			writer.response(400, "Access-Control-Allow-Origin: *");
			JSONObject error = new JSONObject();
			error.put("username_valid", pseudoAvailable);
			error.put("email_valid", emailAvailable);
			writer.write(error.toJSONString());
		}
	}
	
	private String generateGitKey(String email, String pseudo, String password) throws Exception{
		KeyPair pair = generator.generateKeyPair(); //doit être unique ???
		
		JSONObject createUser = new JSONObject();
		createUser.put("email", email);
		createUser.put("username", pseudo);
		createUser.put("password", password);
		post("https://git-users.peerat.dev/api/v1/admin/users/", createUser);
		
		JSONObject sendKey = new JSONObject();
		sendKey.put("key", new String(encoder.encode(pair.getPrivate().getEncoded()))); //add ssh-rsa au début ?
		sendKey.put("read_only", false);
		sendKey.put("title", "peer_at_code_auto_push_key_"+pseudo);
		post("https://git-users.peerat.dev/api/v1/admin/users/"+pseudo+"/keys", sendKey);
		
		return new String(encoder.encode(pair.getPrivate().getEncoded()));
	}
	
	private void post(String url, JSONAware json) throws Exception{
		HttpsURLConnection connection = (HttpsURLConnection) new URL(url).openConnection();
		connection.setRequestMethod("POST");
		connection.setRequestProperty("Content-Type","application/json");
		connection.setRequestProperty("Authorization","Bearer "+this.gitToken);
		connection.setDoInput(true);
		connection.setDoOutput(true);
		
		BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(connection.getOutputStream()));
		writer.write(json.toJSONString());
		writer.flush();
		writer.close();
		
		int response = connection.getResponseCode();
		if(response != 201) throw new Exception("Call to "+url+" failed with response code "+response);
	}

	private void createFolderToSaveSourceCode(String pseudo) throws IOException {

		Files.createDirectories(Paths.get(String.format("%s/%s", usersFilesPath, pseudo)));
	}

}
